// Copyright 1998-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

extern "C" {
#import "OAIQuery.h"
#import "OAIIndex.h"
}

#import <Foundation/Foundation.h>
#import <AIAT/UFSStorage.h>
#import <AIAT/InvertedIndex.h>
#import <AIAT/InvertedAccessor.h>
#import <AIAT/UFSDirectoryCorpus.h>
#import <AIAT/SimpleAnalysis.h>
#import <AIAT/IAAccessor.h>
#import <AIAT/UFSDirectoryCorpus.h>
#import <OmniBase/rcsid.h>

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/OmniAIAT/OAIQuery.M,v 1.8 1999/06/02 06:18:48 bungi Exp $")

NSString *OAIInvalidQueryException = @"OAIInvalidQueryException";

@implementation OAIQuery

static bool ReportQueryProgress(RankedProgress *progress, void *data)
{
    return false;
}

- initWithQuery:(NSString *)query onIndices:(NSArray *)indices maxHits:(unsigned int)max;
{
    InvertedAccessor *accessor;
    unsigned int indexListIndex;
    InvertedIndex **indexList;

    if (![super init])
        return nil;

    indexListIndex = [indices count];
    if (max == 0 || indexListIndex == 0) {
        [self release];
        return nil;
    }

    hitList = NSZoneMalloc(NULL, sizeof(RankedHit *) * max);
    indexList = NSZoneMalloc(NULL, sizeof(InvertedIndex *) * indexListIndex);
    while (indexListIndex--)
        indexList[indexListIndex] = (InvertedIndex *)[[indices objectAtIndex: indexListIndex] indexObject];
    accessor = new InvertedAccessor(indexList, [indices count]);
    accessor->Initialize();

    // AIAT doesn't handle Unicode.  Don't allow Unicode queries either.
    // -cString uses NSMacOSRomanStringEncoding these days.
    if (![query canBeConvertedToEncoding: NSMacOSRomanStringEncoding])
        [NSException raise: OAIInvalidQueryException
                    format: @"AIAT doesn't support Unicode yet.  Please don't use Unicode query strings."];
    
    // I couldn't find documentation for the various types of C++ exceptions which can be thrown here, so I just used a generic handler and raise an NSException assuming that the query format was invalid.
    try {
        resultCount = accessor->RankedSearchBoolean((byte *)[query cString], [query cStringLength], (RankedHit **)hitList, max, &ReportQueryProgress, 999, NULL);
    } catch (...) {
        [NSException raise:OAIInvalidQueryException format:@"A C++ exception was thrown in accessor->RankedSearchBoolean"]; 
    }

    NSZoneFree(NULL, indexList);
    delete accessor;

    return self;
}

- (unsigned int)resultCount;
{
    return resultCount;
}

- (float)scoreAtIndex:(unsigned int)index;
{
    RankedHit *hit;

    hit = hitList[index];
    return hit->GetScore();
}

- (NSString *)filenameAtIndex:(unsigned int)index;
{
    RankedHit *hit;
    UFSDirectoryDoc *doc;

    hit = hitList[index];
    doc = (UFSDirectoryDoc *)hit->GetDocument();
    return [NSString stringWithCString:(char *)(doc->GetFileName())];
}

- (NSString *)pathAtIndex:(unsigned int)index;
{
    RankedHit *hit;
    UFSDirectoryDoc *doc;
    UFSDirectoryCorpus *corpus;
    unsigned char *docName, *docPath;
    NSString *result;
    uint32 length;

    hit = hitList[index];
    doc = (UFSDirectoryDoc *)hit->GetDocument();
    docName = doc->GetFileName();
    corpus = (UFSDirectoryCorpus *)hit->GetIndex()->GetCorpus();
    docPath = corpus->GetDirectory(doc->GetDirectoryID(), &length);
    result = [NSString stringWithCString:(char *)docPath];
    result = [result stringByAppendingPathComponent:[NSString stringWithCString:(char *)docName]];
    return result;
}

- (void)dealloc;
{
    unsigned int resultIndex;

    for (resultIndex = 0; resultIndex < resultCount; resultIndex++)
        delete (RankedHit *)hitList[resultIndex];
    NSZoneFree(NULL, hitList);

    [super dealloc];
}

@end
